
; *
; * GB Camera Screen Capture
; *  by Jeff Frohwein
; *
; * Compiled with RGBASM.
; *

RGBSet:         MACRO
        DW      ((\3 >> 3) << 10) + ((\2 >> 3) << 5) + (\1 >> 3)
        ENDM

        INCLUDE "addsub1.inc"
        INCLUDE "hardware.inc"

        SECTION "High Ram",HRAM

SerSend:        DB

        SECTION "Low Ram",BSS

PacketLen:      DW
Test:           DB
SerState:       DB
SerPtr:         DW

SerBuf:         DS      4096


        SECTION "Org $0",HOME
        ret

        SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

        nop
        jp      Start

        NINTENDO_LOGO                   ; Nintendo graphic logo

;Rom Header Info

;    0123456789ABCDEF
 DB "GB CAM SCRN CAP",0     ; Cart name   16bytes
 DB 0,0,0                     ; Not used
 DB CART_ROM_MBC1             ; Cart type
 DB CART_ROM_256K             ; ROM Size (in bits)
 DB CART_RAM_NONE             ; RAM Size (in bits)
 DB 1,$33                     ; Maker ID
 DB 0                         ; Version     =0
 DB $e2                       ; Complement check (important)
 DW $c40e                     ; Checksum (not important)

        INCLUDE "memory1.asm"   ; memory set/copy

Start:
        di
        ld      sp,$fffe

        ld      a,0
        ld      [rIE],a

; * Display a picture *

        call    DisplayOff

        ld      hl,$c000
        ld      de,$8000
        ld      bc,$1000
        call    mem_Copy        ; Copy tileset to screen

        ld      hl,PicMap
        ld      de,$9800
        call    Copy20x18       ; Copy tile map to memory

        ld      a,$e4
        ld      [rBGP],a        ; Setup default background colors

        xor     a
        ld      [rSCX],a
        ld      [rSCY],a

        ld      a,LCDCF_ON|LCDCF_BG8000|LCDCF_BG9800|LCDCF_BGON|LCDCF_OBJ16|LCDCF_OBJOFF
        ld      [rLCDC],a         ; Turn screen on

trap:   jr      trap


; Copy char map to screen
; Entry: HL = Source Map
;        DE = Destination Screen

Copy20x18:
        ld      b,18
Copy20xB:
        ld      c,20
Copy202:
        ld      a,[hl+]
        ld      [de],a
        inc     de
        dec     c
        jr      nz,Copy202

        add16ir de,12

        dec     b
        jr      nz,Copy20xB

        ret

; *** Turn off LCD Display ***

DisplayOff:
        ld      hl,rLCDC
        bit     7,[hl]          ; Is LCD already off?
        ret     z               ; yes, exit

        ld      a,[rIE]
        push    af
        res     0,a
        ld      [rIE],a         ; Disable vblank interrupt if enabled

.loop:  ld      a,[rLY]         ; Loop until in first part of vblank
        cp      145
        jr      nz,.loop

        res     7,[hl]          ; Turn the screen off

        pop     af
        ld      [rIE],a         ; Restore the state of vblank interrupt
        ret

FLB     equ     $ff

PicMap:
        db      FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB
        db      FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB
        db      FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB
        db      FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB
        db      FLB,FLB,$00,$01,$02,$03,$04,$05,$06,$07
        db      $08,$09,$0a,$0b,$0c,$0d,$0e,$0f,FLB,FLB
        db      FLB,FLB,$10,$11,$12,$13,$14,$15,$16,$17
        db      $18,$19,$1a,$1b,$1c,$1d,$1e,$1f,FLB,FLB
        db      FLB,FLB,$20,$21,$22,$23,$24,$25,$26,$27
        db      $28,$29,$2a,$2b,$2c,$2d,$2e,$2f,FLB,FLB
        db      FLB,FLB,$30,$31,$32,$33,$34,$35,$36,$37
        db      $38,$39,$3a,$3b,$3c,$3d,$3e,$3f,FLB,FLB
        db      FLB,FLB,$40,$41,$42,$43,$44,$45,$46,$47
        db      $48,$49,$4a,$4b,$4c,$4d,$4e,$4f,FLB,FLB
        db      FLB,FLB,$50,$51,$52,$53,$54,$55,$56,$57
        db      $58,$59,$5a,$5b,$5c,$5d,$5e,$5f,FLB,FLB
        db      FLB,FLB,$60,$61,$62,$63,$64,$65,$66,$67
        db      $68,$69,$6a,$6b,$6c,$6d,$6e,$6f,FLB,FLB
        db      FLB,FLB,$70,$71,$72,$73,$74,$75,$76,$77
        db      $78,$79,$7a,$7b,$7c,$7d,$7e,$7f,FLB,FLB
        db      FLB,FLB,$80,$81,$82,$83,$84,$85,$86,$87
        db      $88,$89,$8a,$8b,$8c,$8d,$8e,$8f,FLB,FLB
        db      FLB,FLB,$90,$91,$92,$93,$94,$95,$96,$97
        db      $98,$99,$9a,$9b,$9c,$9d,$9e,$9f,FLB,FLB
        db      FLB,FLB,$a0,$a1,$a2,$a3,$a4,$a5,$a6,$a7
        db      $a8,$a9,$aa,$ab,$ac,$ad,$ae,$af,FLB,FLB
        db      FLB,FLB,$b0,$b1,$b2,$b3,$b4,$b5,$b6,$b7
        db      $b8,$b9,$ba,$bb,$bc,$bd,$be,$bf,FLB,FLB
        db      FLB,FLB,$c0,$c1,$c2,$c3,$c4,$c5,$c6,$c7
        db      $c8,$c9,$ca,$cb,$cc,$cd,$ce,$cf,FLB,FLB
        db      FLB,FLB,$d0,$d1,$d2,$d3,$d4,$d5,$d6,$d7
        db      $d8,$d9,$da,$db,$dc,$dd,$de,$df,FLB,FLB
        db      FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB
        db      FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB
        db      FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB
        db      FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB,FLB
